package simulation;

import simulation.model.PhysicalObject;
import simulation.model.Sensor;
import simulation.model.Simulation;
import simulation.results.SimulationResult;

import java.util.LinkedList;
import java.util.List;
import java.util.stream.StreamSupport;

public class InfoGatherer implements Simulation.ObjectSeenListener {

    private final Simulation simulation;
    private final int duration;

    private final List<Integer> missedTimestamps = new LinkedList<>();
    private final List<Integer> seenTimestamps = new LinkedList<>();
    private final List<Integer> criticalMisses = new LinkedList<>();

    public InfoGatherer(Simulation simulation, int duration) {
        this.simulation = simulation;
        this.duration = duration;

        this.simulation.setObjectSeenListener(this);
    }

    public SimulationResult run() {
        final var numCycles = duration * 1000;

        seenTimestamps.clear();
        missedTimestamps.clear();

        while (simulation.getMillisCompleted() < numCycles) {
            simulation.cycle();
        }

        return new SimulationResult(
                simulation,
                seenTimestamps,
                missedTimestamps,
                criticalMisses,
                StreamSupport.stream(simulation.getSensors().spliterator(), false)
                        .mapToDouble(Sensor::getEnergyUsed)
                        .sum()
        );
    }

    @Override
    public void onObjectSeen(Simulation simulation, PhysicalObject object) {
        seenTimestamps.add(simulation.getMillisCompleted());
    }

    @Override
    public void onObjectMissed(Simulation simulation, PhysicalObject object) {
        missedTimestamps.add(simulation.getMillisCompleted());

        var hitsPerson = object.getPosition().x > -object.width - simulation.getPerson().width / 2 &&
                        object.getPosition().y < simulation.getPerson().width / 2;

        if (hitsPerson) {
            criticalMisses.add(simulation.getMillisCompleted());
        }
    }
}
