package simulation.gui;

import simulation.model.Simulation;
import simulation.visualization.SimulationPainter;

import javax.swing.*;
import java.awt.*;

public class SimulationPanel extends JPanel implements Simulation.SimulationCycleListener {

    private static final float SCALE = 1;

    private final SimulationPainter simulationPainter;
    private Simulation simulation;

    public SimulationPanel() {
        super();
        simulationPainter = new SimulationPainter();

        setBackground(Color.GRAY);
    }

    public void setSimulation(Simulation simulation) {
        if (this.simulation != null) {
            this.simulation.clearCycleListener();
        }

        this.simulation = simulation;
        this.simulation.setCycleListener(this);
        repaint();
    }

    public Simulation getSimulation() {
        return simulation;
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2d = (Graphics2D) g;

        if (simulation == null) {
            paintNoSimulation(g2d);
        } else {
            paintSimulation(g2d);
        }
    }

    private void paintSimulation(Graphics2D g2d) {
        simulationPainter.paint(simulation, g2d, getWidth(), getHeight(), SCALE);
    }

    private void paintNoSimulation(Graphics2D g2d) {
        final var text = "No simulation yet.";
        final var textWidth = g2d.getFontMetrics().stringWidth(text);

        final var x = (getWidth() - textWidth) / 2;
        final var y = (getHeight() / 2) - g2d.getFontMetrics().getDescent();

        g2d.drawString("No simulation yet.", x, y);
    }

    @Override
    public void onSimulationCycle() {
        repaint();
    }
}
