package simulation.gui;

import simulation.model.Simulation;

import javax.swing.*;
import java.awt.*;

public final class Window {

    private JFrame frame;

    private SimulationPanel simulationPanel;
    private ControlPanel controlPanel;

    public Window() {
        initialize();
    }

    private void initialize() {
        frame = new JFrame("Robots Everywhere: Sensor simulation");
        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

        simulationPanel = new SimulationPanel();
        simulationPanel.setPreferredSize(new Dimension(700, 700));

        controlPanel = new ControlPanel();
        controlPanel.setPreferredSize(new Dimension(400, 500));

        frame.setLayout(new BorderLayout());
        frame.add(simulationPanel, BorderLayout.CENTER);
        frame.add(controlPanel, BorderLayout.LINE_START);

        controlPanel.setLoadListener(() -> {
            var simulation = new Simulation(
                    controlPanel.getNumSensors(),
                    controlPanel.getRotationSpeed(),
                    controlPanel.getRotationAmount(),
                    controlPanel.getIntervals()
            );

            simulationPanel.setSimulation(simulation);
        });

        controlPanel.setCycleRequestedListener(() -> {
            var simulation = simulationPanel.getSimulation();

            if (simulation != null) {
                simulationPanel.getSimulation().cycle();
            }
        });

        frame.pack();
        frame.setVisible(true);
    }

}
