package simulation.results;

import com.opencsv.CSVWriter;

import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;

public class CsvResultPrinter implements NamedSimulationResultPrinter {

    private final CSVWriter csvWriter;

    public CsvResultPrinter() throws IOException {
        final var filePath = Paths.get("results.csv");

        if (Files.exists(filePath)) {
            Files.delete(filePath);
        }

        Files.createFile(filePath);

        csvWriter = new CSVWriter(new FileWriter(filePath.toFile()));
        csvWriter.writeNext(new String[] {
                "Number of sensors",
                "Rotation per second (degrees/second)",
                "Rotation amount (degrees)",
                "Objects missed",
                "Critical misses (would hit person)",
                "Objects seen",
                "Percentage missed",
                "Energy used (mAh)"
        });
    }

    @Override
    public String getName() {
        return "CSV";
    }

    @Override
    public void collect(SimulationResult result) {
        csvWriter.writeNext(buildRow(result));
    }

    private String[] buildRow(SimulationResult result) {
        final var simulation = result.simulation;
        final var missedTimestamps = result.missedTimestamps;
        final var seenTimestamps = result.seenTimestamps;

        return new String[] {
                String.valueOf(simulation.getNumSensors()),
                String.valueOf(simulation.getRotationPerSecond()),
                String.valueOf(simulation.getRotationAmount()),
                String.valueOf(missedTimestamps.size()),
                String.valueOf(result.percentageCriticalMiss()),
                String.valueOf(seenTimestamps.size()),
                String.valueOf(result.percentageMissed()),
                String.valueOf(result.energyUsed)
        };
    }

    @Override
    public void print() {
        try {
            csvWriter.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
