package simulation.results;

public class SystemOutSimulationResultPrinter implements NamedSimulationResultPrinter {

    @Override
    public String getName() {
        return "System.out";
    }

    @Override
    public void collect(SimulationResult result) {
        var simulation = result.simulation;
        var seenTimestamps = result.seenTimestamps;
        var missedTimestamps = result.missedTimestamps;

        System.out.println("Simulation");
        System.out.println(" - Cycles executed: " + simulation.getMillisCompleted());
        System.out.println(" - Intervals: " + simulation.hasIntervals());
        System.out.println(" - Number of sensors: " + simulation.getNumSensors());
        System.out.println(" - Rotation speed: " + simulation.getRotationPerSecond() + " degrees/second");
        System.out.println(" - Rotation amount: " + simulation.getRotationAmount() + " degrees");

        System.out.println("\nResults:");
        System.out.println(" - Objects seen: " + seenTimestamps.size());
        System.out.println(" - Objects missed: " + missedTimestamps.size());

        System.out.println(String.format(" - Percentage missed: %.4f%%", result.percentageMissed()));
    }

    @Override
    public void print() {
    }
}
