package simulation.visualization;

import simulation.model.PhysicalObject;
import simulation.model.Simulation;
import simulation.util.Vector2d;

import java.awt.*;
import java.awt.geom.Ellipse2D;

public class SimulationPainter {
    public void paint(Simulation simulation, Graphics2D g2d, int width, int height, double scale) {
        g2d.translate(width / 2, height - 10);
        g2d.scale(scale, scale);

        // Draw the body of the person
        final var body = getBodyEllipse(simulation);
        g2d.setStroke(new BasicStroke());
        g2d.setColor(Color.BLACK);
        g2d.draw(body);

        // Draw the objects
        for (PhysicalObject obj : simulation.getObjects()) {
            var pos = obj.getPosition();

            g2d.setColor(obj.isSeen() ? Color.RED : Color.WHITE);
            g2d.fillRect((int) pos.x, (int) pos.y, obj.width, obj.height);

            g2d.setColor(Color.BLACK);
            g2d.drawRect((int) pos.x, (int) pos.y, obj.width, obj.height);
        }

        // Draw the sensor
        g2d.setColor(Color.GREEN);
        for (var sensor : simulation.getSensors()) {
            final var baseRotation = Math.toRadians(sensor.getBaseRotation() - 90);
            final var offsetX = Math.cos(baseRotation) * getScaledPersonHeight(simulation);
            final var offsetY = Math.sin(baseRotation) * getScaledPersonHeight(simulation);

            final var arc = sensor.toArc(new Vector2d((float) offsetX, (float) offsetY));

            g2d.setColor(Color.GREEN);
            g2d.fill(arc);

            g2d.setColor(Color.BLACK);
            g2d.draw(arc);
        }

        g2d.setColor(Color.WHITE);
        g2d.drawString(String.format("%.2f s", simulation.getMillisCompleted() / 1000.0f), -width / 2 + 10, -height + 30);
    }

    private float getScaledPersonHeight(Simulation simulation) {
        return simulation.getPerson().depth / 100f;
    }

    private Ellipse2D.Double getBodyEllipse(Simulation simulation) {
        final var width = getScaledPersonHeight(simulation);

        final var x = -width / 2;
        final var y = -width / 2;

        return new Ellipse2D.Double(x, y, width, width);
    }
}
